@@ -1,12 +1,12 @@
 r""":mod:`range_streams.stream` exposes a class
-:py:func:`RangeStream`, whose key property (once initialised) is
+:class:`~range_streams.stream.RangeStream`, whose key property (once initialised) is
 :attr:`~range_streams.stream.RangeStream.ranges`,
 which provides a :class:`~ranges.RangeDict` comprising the ranges of
 the file being streamed.
 
-The method :py:func:`RangeStream.add` will request further ranges,
-and (unlike the other methods in this module) will accept
-a tuple of two integers as its argument (``byte_range``).
+The method :meth:`~range_streams.stream.RangeStream.add` will request further ranges,
+and (unlike the other methods in this module) will accept a tuple of two integers as its
+argument (``byte_range``).
 """
 
 from __future__ import annotations
@@ -14,7 +14,7 @@ from __future__ import annotations
 from copy import deepcopy
 from io import SEEK_SET
 from pathlib import Path
-from typing import TYPE_CHECKING
+from typing import TYPE_CHECKING, Callable, Coroutine, Type
 from urllib.parse import urlparse
 
 MYPY = False  # when using mypy will be overrided as True
@@ -23,9 +23,11 @@ if MYPY or not TYPE_CHECKING:  # pragma: no cover
 
 from ranges import Range, RangeDict
 
-from .http_utils import detect_header_value
+from .async_utils import AsyncFetcher
+from .http_utils import detect_header_value, range_header
 from .overlaps import get_range_containing, overlap_whence
 from .range_utils import (
+    ALWAYS_SET_TOLD,
     most_recent_range,
     range_max,
     range_span,
@@ -38,6 +40,8 @@ from .response import RangeResponse
 
 __all__ = ["RangeStream"]
 
+DEBUG_VERBOSE = False
+
 
 class RangeStream:
     """
@@ -58,6 +62,10 @@ class RangeStream:
     (an external requirement installed alongside this package). Either
     way, the interval created is interpreted to be the standard Python
     convention of a half-open interval ``[start,stop)``.
+
+    Don't forget to close the ``httpx.Response`` yourself! The
+    :meth:`~range_streams.stream.RangeStream.close` method is available
+    (or :meth:`~range_streams.response.RangeResponse.close`) to help you.
     """
 
     _length_checked: bool = False
@@ -76,6 +84,14 @@ class RangeStream:
     reading in bytes from the :class:`RangeResponse` (unlike the
     'external' :attr:`~range_streams.stream.RangeStream.ranges` property)
     """
+    _range_windows: RangeDict
+    """
+    `'Internal'` ranges attribute for windowed ranges. Start position is not affected by
+    reading in bytes from the :class:`RangeResponse` (unlike the 'external'
+    :attr:`~range_streams.stream.RangeStream.ranges` property). Used in single request
+    mode (when :attr:`~range_streams.stream.RangeStream.single_request` is set to
+    ``True`` at initialisation).
+    """
 
     def __init__(
         self,
@@ -83,7 +99,9 @@ class RangeStream:
         client=None,  # don't hint httpx.Client (Sphinx gives error)
         byte_range: Range | tuple[int, int] = Range("[0, 0)"),
         pruning_level: int = 0,
+        single_request: bool = False,
         force_async: bool = False,
+        chunk_size: int | None = None,
     ):
         """
         Set up a stream for the file at ``url``, with either an initial
@@ -101,32 +119,72 @@ class RangeStream:
         either way as a half-closed interval ``[start, end)``, as given by
         Python's built-in :class:`range`.
 
+        If ``byte_range`` is passed as the empty range ``Range(0,0)`` (its default),
+        then a HEAD request is sent to ``url`` on initialisation, setting the
+        :attr:`~range_streams.stream.RangeStream.total_bytes` value from the
+        ``content-length`` header in the subsequent response.
+
+        If ``single_request`` is ``True`` (default: ``False``), then the behaviour when
+        an empty ``byte_range`` is passed instead becomes to send a standard streaming
+        GET request (not a partial content request at all), and instead the class will
+        then facilitate an interface that 'simulates' these calls, i.e. as if each time
+        :meth:`~range_streams.stream.RangeStream.add` was used the range requests were
+        being returned instantly (as everything needed was already obtained on the first
+        request at initialisation). More performant when reading a stream linearly.
+
+        Note: internally, this single request is known as 'the monostream', and is
+        stored on the :attr:`~range_streams.stream.RangeStream.monostream` property.
+
+        Note: a single request will not be as efficient if streaming the response
+        non-linearly (since reading a byte in the stream requires loading all bytes up
+        to it). This will mean it is only performant to use for certain file types or
+        applications (e.g. a ZIP file is read "in a principled manner" from the end
+        [the Central Directory] first, so gains greatly from using multiple partial
+        content requests rather than a single stream, whereas a PNG file can only be
+        read "in a principled manner" linearly, iterating through the chunks from the
+        start).
+
         The ``pruning_level`` controls the policy for overlap handling
         (``0`` will resize overlapped ranges, ``1`` will delete overlapped
         ranges, and ``2`` will raise an error when a new range is added
         which overlaps a pre-existing range).
 
+        The ``chunk_size`` controls the size of the chunks that are read in from the
+        ``httpx.Response.iter_raw`` iterator on the streamed HTTP response.
+
         - See docs for the
           :meth:`~range_streams.stream.RangeStream.handle_overlap`
           method for further details.
 
         Args:
-          url           : (:class:`str`) The URL of the file to be streamed
-          client        : (:class:`httpx.Client` | ``None``) The HTTPX client
-                          to use for HTTP requests
-          byte_range    : (:class:`~ranges.Range` | ``tuple[int,int]``) The range
-                          of positions on the file to be requested
-          pruning_level : (:class:`int`) Either ``0`` ('replant'), ``1`` ('burn'),
-                          or ``2`` ('strict')
-          force_async   : (:class:`bool` | ``None``) Whether to require the client
-                          to be ``httpx.AsyncClient``, and if no client is given,
-                          to create one on initialisation.
+          url            : (:class:`str`) The URL of the file to be streamed
+          client         : (:class:`httpx.Client` | ``None``) The HTTPX client
+                           to use for HTTP requests
+          byte_range     : (:class:`~ranges.Range` | ``tuple[int,int]``) The range
+                           of positions on the file to be requested
+          pruning_level  : (:class:`int`) Either ``0`` ('replant'), ``1`` ('burn'),
+                           or ``2`` ('strict')
+          single_request : (:class:`bool`) Whether to use a single GET request and
+                           just add 'windows' onto this rather than create multiple
+                           partial content requests.
+          force_async    : (:class:`bool` | ``None``) Whether to require the client
+                           to be ``httpx.AsyncClient``, and if no client is given,
+                           to create one on initialisation. (Experimental/WIP)
+          chunk_size     : (:class:`int` | ``None``) The chunk size used for the
+                            ``httpx.Response.iter_raw`` response byte iterators
         """
         self.url = url
         self.set_client(client=client, force_async=force_async)
         self.pruning_level = pruning_level
+        self.single_request = single_request
+        self.chunk_size = chunk_size
         self._ranges = RangeDict()
-        self.add(byte_range=byte_range)
+        self._range_windows = RangeDict()
+        if self.client_is_async:
+            # await self.async_add(byte_range=byte_range)
+            pass  # Can't call async_add from a synchronous init method
+        else:
+            self.add(byte_range=byte_range)
 
     def __repr__(self) -> str:
         return (
@@ -134,6 +192,15 @@ class RangeStream:
             f"'{self.name}' from {self.domain}"
         )
 
+    @property
+    def freely_requestable(self):
+        """
+        Trivial opposite of the :attr:`~range_streams.stream.RangeStream.single_request`
+        attribute, so that conditional blocks can treat this as the 'conventional' case
+        and the single request case be the alternative (which looks better).
+        """
+        return not self.single_request
+
     def set_client(self, client, force_async: bool) -> None:
         """
         Check client type explicitly to handle a/sync and optional HTTPX client.
@@ -178,17 +245,18 @@ class RangeStream:
         if rng not in self.total_range:
             raise ValueError(f"{rng} is not a sub-range of {self.total_range}")
 
-    def check_range_integrity(self) -> None:
+    def check_range_integrity(self, use_windows=False) -> None:
         """
         Every :class:`~ranges.RangeSet` in the
         :attr:`~range_streams.stream.RangeStream._ranges`
         :class:`~ranges.RangeDict` keys must contain 1 :class:`~ranges.Range` each
         """
-        if sum(len(rs._ranges) - 1 for rs in self._ranges.ranges()) != 0:
-            bad_rs = [rs for rs in self._ranges.ranges() if len(rs._ranges) - 1 != 0]
+        rng_dict = self._range_windows if use_windows else self._ranges
+        if sum(len(rs._ranges) - 1 for rs in rng_dict.ranges()) != 0:
+            bad_rs = [rs for rs in rng_dict.ranges() if len(rs._ranges) - 1 != 0]
             for rset in bad_rs:
                 for rng in rset:
-                    rng_resp = self._ranges[rng.start]
+                    rng_resp = rng_dict[rng.start]
                     rng_max = range_max(rng)
                     if rng_resp.tell() > rng_max:
                         rset.discard(rng)  # discard subrange
@@ -197,23 +265,30 @@ class RangeStream:
             if bad_rs:
                 raise ValueError(f"Each RangeSet must contain 1 Range: found {bad_rs=}")
 
-    def compute_external_ranges(self) -> RangeDict:
+    def compute_external_ranges(self, use_windows: bool = False) -> RangeDict:
         """
-        Modifying the :attr:`~range_streams.stream.RangeStream._ranges`
-        attribute to account for the bytes consumed (from the head)
-        and tail mark offset of where a range was already trimmed to avoid
-        an overlap (from the tail).
+        If ``use_windows`` is ``True``, the ``internal_range_dict``
+        is :attr:`~range_streams.stream.RangeStream._range_windows`
+        rather than :attr:`~range_streams.stream.RangeStream._ranges`
+        when ``use_windows`` is ``False`` (default: ``False``).
+
+        Modifying the ``internal_range_dict`` attribute to account for the bytes
+        consumed (from the head) and tail mark offset of where a range was already
+        trimmed to avoid an overlap (from the tail).
 
         While the :class:`~ranges.RangeSet` keys are a deep copy of the
-        :attr:`~range_streams.stream.RangeStream._ranges`
-        :class:`~ranges.RangeDict` keys (and therefore will not propagate if modified),
-        the RangeResponse values are references, therefore will propagate to the
-        :attr:`~range_streams.stream.RangeStream._ranges`
-        :class:`~ranges.RangeDict` if modified.
+        ``internal_range_dict`` :class:`~ranges.RangeDict` keys (and therefore will not
+        propagate if modified), the RangeResponse values are references, therefore will
+        propagate to the ``internal_range_dict`` :class:`~ranges.RangeDict` if modified
+        (primarily when ``read``).
+
+        When ``use_windows`` is ``True``, these RangeResponse values are 'simulations'
+        (a.k.a. mock/dummy objects) of the range response that would be received from a
+        partial content request (they in fact merely came from a streamed GET request).
         """
         prepared_rangedict = RangeDict()
-        internal_rangedict = self._ranges.items()
-        for rng_set, rng_response in internal_rangedict:
+        internal_rangedict = self._range_windows if use_windows else self._ranges
+        for rng_set, rng_response in internal_rangedict.items():
             requested_range = rng_response.request.range
             rng = deepcopy(requested_range)
             # if (rng_response.start, rng_response.end) < 0:
@@ -221,12 +296,17 @@ class RangeStream:
             #    ...
             if rng_response.is_consumed():
                 continue
-            if rng_response_tell := rng_response.tell():
+            told_is_set = rng_response.is_windowed or ALWAYS_SET_TOLD
+            if rng_response_tell := (
+                rng_response.told if told_is_set else rng_response.tell()
+            ):
                 # Access single range (assured by unique RangeResponse values of
                 # RangeDict) of singleton rangeset (assured by check_range_integrity)
                 rng.start += rng_response_tell
             if rng_response.tail_mark:
                 rng.end -= rng_response.tail_mark
+            if rng.start > rng.end:
+                raise ValueError(f"{rng} has been malformed (rng_response=)")
             prepared_rangedict.update({rng: rng_response})
         return prepared_rangedict
 
@@ -240,32 +320,64 @@ class RangeStream:
 
         Each :attr:`~range_streams.stream.RangeStream.ranges` :class:`~ranges.RangeDict`
         key is a :class:`~ranges.RangeSet` containing 1 :class:`~ranges.Range`. Check
-        this assumption (singleton :class:`~ranges.RangeSet` "integrity") holds and retrieve
-        this list of :class:`~ranges.RangeSet` keys in ascending order, as a list of
-        :class:`~ranges.Range`.
+        this assumption (singleton :class:`~ranges.RangeSet` "integrity") holds and
+        retrieve this list of :class:`~ranges.RangeSet` keys in ascending order, as a
+        list of :class:`~ranges.Range`.
+
+        Requests are restricted to not re-request already-requested file ranges, so give
+        windows onto the underlying range that can be consumed (but the underlying
+        :class:~range_streams.response.RangeResponse` will persist and cannot be
+        consumed by reading).
         """
         self.check_range_integrity()
-        return self.compute_external_ranges()
+        # Unclear if this is necessary but seems consistent to do here:
+        if self.single_request:
+            # Also check integrity of the range windows
+            self.check_range_integrity(use_windows=True)
+        if self.freely_requestable:
+            # Not limited to a single request
+            ranges = self.compute_external_ranges()
+        else:
+            # Single request limit: can only add a window onto already requested range
+            ranges = self.compute_external_ranges(use_windows=True)
+        return ranges
 
-    def overlap_whence(self, rng: Range, internal: bool = False) -> int | None:
-        rng_dict = self._ranges if internal else self.ranges
-        return overlap_whence(rng_dict, rng)
+    def overlap_whence(
+        self, rng: Range, internal: bool = False, use_windows: bool = False
+    ) -> int | None:
+        if DEBUG_VERBOSE:
+            print(f"IN {rng=} {internal=} {use_windows=}")
+            for k, v in self._range_windows.items():
+                print(f"{k}: {v} ({v.told})")
+        internal_rng_dict = self._range_windows if use_windows else self._ranges
+        rng_dict = internal_rng_dict if internal else self.ranges
+        if DEBUG_VERBOSE:
+            print(f"OUT {rng}")
+        return overlap_whence(rng_dict=rng_dict, rng=rng)
 
     def register_range(
         self,
         rng: Range,
         value: RangeResponse,
         activate: bool = True,
+        use_windows: bool = False,
     ):
         if self._length_checked:
             self.check_is_subrange(rng)
         else:
             raise ValueError("Stream length must be set before registering a range")
-        if self.overlap_whence(rng, internal=False) is not None:
-            self.handle_overlap(rng, internal=False)
+        if DEBUG_VERBOSE:
+            print(f"Hit {rng=} {value=} {activate=} {use_windows=}")
+        if (
+            self.overlap_whence(rng, internal=False, use_windows=use_windows)
+            is not None
+        ):
+            self.handle_overlap(rng, internal=False, use_windows=use_windows)
         # print(f"Pre: {self._ranges=}")
         # print(f"Adding: {rng=}")
-        self._ranges.add(rng=rng, value=value)
+        ranges = self._range_windows if use_windows else self._ranges
+        # This is where a previous tail mark is erased (if replacing an overlap)
+        ranges.add(rng=rng, value=value)
         if activate:
             self.set_active_range(rng)
         # print(f"Post: {self._ranges=}")
@@ -288,9 +400,13 @@ class RangeStream:
         :class:`~ranges.Range` key for the internal
         :attr:`~range_streams.stream.RangeStream._ranges`
         :class:`RangeDict`.
+
+        Look it up in the :attr:`~range_streams.stream.RangeStream._ranges`
+        :class:`RangeDict` instead if in single request mode.
         """
+        internal_rng_dict = self._range_windows if self.single_request else self._ranges
         try:
-            return self._ranges[self._active_range]
+            return internal_rng_dict[self._active_range]
         except KeyError:
             e_pre = "Cannot get active range response "
             if self._active_range is None:
@@ -340,6 +456,7 @@ class RangeStream:
         self,
         rng: Range,
         internal: bool = False,
+        use_windows: bool = False,
     ) -> None:
         """
         Handle overlaps with a given pruning level:
@@ -349,7 +466,8 @@ class RangeStream:
         1. "burn" existing ranges overlapped anywhere by the new range
         2. "strict" will throw a :class:`ValueError`
         """
-        ranges = self._ranges if internal else self.ranges
+        internal_rng_dict = self._range_windows if use_windows else self._ranges
+        ranges = internal_rng_dict if internal else self.ranges
         if self.pruning_level not in range(3):
             raise ValueError("Pruning level must be 0, 1, or 2")
         # print(f"Handling {rng=} with {self.pruning_level=}")
@@ -457,6 +575,112 @@ class RangeStream:
             byte_range=byte_range,
             url=self.url,
             client=self.client,
+            chunk_size=self.chunk_size,
+        )
+
+    def simulate_request(
+        self, byte_range: Range, parent_range_request: RangeRequest | None = None
+    ) -> RangeRequest:
+        """
+        Simulate the :class:`~range_streams.request.RangeRequest` obtained from a
+        partial content request for ``byte_range`` on the stream's URL through a
+        "window" on ``range_request`` (expected to be a streamed GET request for the
+        full file range).
+
+        If no ``parent_range_request`` is provided, it is assumed to be the one on the
+        :class:`~range_streams.response.RangeResponse` in the internal
+        :attr:`~range_streams.stream.RangeStream._ranges` :class:`~ranges.RangeDict`
+
+        Args:
+          byte_range           : The :class:`~ranges.Range` to simulate a partial
+                                 content request for.
+          parent_range_request : The :class:`~range_streams.request.RangeRequest` over
+                                 which to use a "window" to simulate the range request.
+        """
+        if parent_range_request is None:
+            access_pos = byte_range.start
+            if access_pos not in self._ranges:
+                msg = f"{access_pos} is not in internal RangeDict\n{self._ranges=}"
+                raise ValueError(msg)
+            parent_range_response = self._ranges[access_pos]
+            parent_range_request = parent_range_response.request
+        return RangeRequest.windowed_request(
+            byte_range=byte_range,
+            range_request=parent_range_request,
+            tail_mark=parent_range_response.tail_mark,
+            chunk_size=self.chunk_size,
+        )
+
+    async def get_async_monostream(self) -> None:
+        """
+        Send a streaming GET request with an open-ended ``content-range`` header, to
+        obtain the total range. Suitable for higher performance (to avoid repeated
+        requests on the :class:`~range_streams.stream.RangeStream` which accrue a time
+        cost).
+
+        Should be called after the :class:`~range_streams.stream.RangeStream` is
+        initialised (with both ``single_request`` and ``force_async`` as True), and
+        [unlike the initialisation method] of course this method must be awaited.
+        """
+        rng_h = range_header(rng=Range(0, 0))  # Empty range -> open-ended range header
+        req = self.client.build_request(method="GET", url=self.url, headers=rng_h)
+        resp = await self.client.send(request=req, stream=True)
+        resp.raise_for_status()
+        total_length = self.check_response_length(headers=resp.headers, req=req.method)
+        self.set_length(length=total_length)
+
+        # This is where self.send_request would give a RangeRequest...
+        range_req = RangeRequest.from_get_stream(
+            byte_range=self.total_range,
+            client=self.client,
+            req=req,
+            resp=resp,
+            chunk_size=self.chunk_size,
+        )
+        await range_req.await_aiterator()  # Initialise its async stream iterator
+
+        # then just use the req to create a RangeResponse and register as usual
+        resp = RangeResponse(stream=self, range_request=range_req, range_name="")
+        self.register_range(
+            rng=self.total_range,
+            value=resp,
+            activate=False,  # Don't set the active range as this is 'internal'
+            use_windows=False,
+        )
+
+    def get_monostream(self) -> None:
+        """
+        Send a streaming GET request with an open-ended ``content-range`` header, to
+        obtain the total range. Suitable for higher performance (to avoid repeated
+        requests on the :class:`~range_streams.stream.RangeStream` which accrue a time
+        cost).
+
+        Called at initialisation (within the first) when ``single_request`` is passed
+        to :class:`~range_streams.stream.RangeStream` as ``True``.
+        """
+        rng_h = range_header(rng=Range(0, 0))  # Empty range -> open-ended range header
+        req = self.client.build_request(method="GET", url=self.url, headers=rng_h)
+        resp = self.client.send(request=req, stream=True)
+        resp.raise_for_status()
+        total_length = self.check_response_length(headers=resp.headers, req=req.method)
+        self.set_length(length=total_length)
+
+        # This is where self.send_request would give a RangeRequest...
+        range_req = RangeRequest.from_get_stream(
+            byte_range=self.total_range,
+            client=self.client,
+            req=req,
+            resp=resp,
+            chunk_size=self.chunk_size,
+        )
+
+        # then just use the req to create a RangeResponse and register as usual
+        resp = RangeResponse(stream=self, range_request=range_req, range_name="")
+        self.register_range(
+            rng=self.total_range,
+            value=resp,
+            activate=False,  # Don't set the active range as this is 'internal'
+            use_windows=False,
         )
 
     def send_head_request(self) -> None:
@@ -469,15 +693,25 @@ class RangeStream:
         If the :attr:`range_streams.stream.RangeStream.client` is asynchronous, use
         a synchronous client (created for this single request).
         """
-        req = self.sync_client.build_request("HEAD", self.url)
+        req = self.sync_client.build_request(method="HEAD", url=self.url)
         resp = self.sync_client.send(request=req)
         resp.raise_for_status()
-        key = "content-length"
-        try:
-            total_length = detect_header_value(headers=resp.headers, key=key)
-        except KeyError as exc:
-            raise KeyError(f"HEAD request response was missing '{key}' header") from exc
-        self.set_length(int(total_length))
+        total_length = self.check_response_length(headers=resp.headers, req=req.method)
+        self.set_length(length=total_length)
+
+    def check_response_length(self, headers: dict[str, str], req: str) -> int:
+        """
+        Return the length of the response from its ``content-length`` header (after
+        checking it contains this header, else raising :class:`KeyError`), as an integer.
+
+        Args:
+          headers : The response headers
+          req     : The request method (to be reported in any :class:`KeyError` raised)
+        """
+        total_length = detect_header_value(
+            headers=headers, key="content-length", source=f"{req} request response"
+        )
+        return int(total_length)
 
     def set_length(self, length: int) -> None:
         self._length = length
@@ -506,6 +740,52 @@ class RangeStream:
         """
         return [rngset.ranges()[0] for rngset in self.ranges.ranges()]
 
+    async def add_async(
+        self,
+        byte_range: Range | tuple[int, int] = Range("[0, 0)"),
+        activate: bool = True,
+        name: str = "",
+    ) -> None:
+        byte_range = validate_range(byte_range=byte_range, allow_empty=True)
+        if not self.single_request:
+            raise NotImplementedError(
+                "Async RangeStreams are only available in single request mode (for now)"
+            )
+        # Do not request an empty range if total length already checked (at init)
+        if not self._length_checked and byte_range.isempty():
+            await self.get_async_monostream()
+        elif not byte_range.isempty():
+            self.add_window(byte_range=byte_range, activate=activate, name=name)
+
+    def add_window(
+        self,
+        byte_range: Range | tuple[int, int] = Range("[0, 0)"),
+        activate: bool = True,
+        name: str = "",
+    ) -> None:
+        """
+        Register a window onto the original range in the ``_ranges`` RangeDict rather
+        than add a new range entry to the dict (which would A) clash with the single
+        entire range B) require another request
+
+        Args:
+          byte_range : (:class:`~ranges.Range` | ``tuple[int,int]``) The range
+                       of positions on the file to be read from the request on
+                       the stream.
+          activate   : (:class:`bool`) Whether to make this newly added
+                       :class:`~ranges.Range` the active range on the stream upon
+                       creating it.
+          name       : (:class:`str`) A name (default: ``''``) to give to the range.
+        """
+        req = self.simulate_request(byte_range=byte_range)
+        resp = RangeResponse(stream=self, range_request=req, range_name=name)
+        self.register_range(
+            rng=byte_range,
+            value=resp,
+            activate=activate,
+            use_windows=True,
+        )
+
     def add(
         self,
         byte_range: Range | tuple[int, int] = Range("[0, 0)"),
@@ -543,22 +823,73 @@ class RangeStream:
                        creating it.
           name       : (:class:`str`) A name (default: ``''``) to give to the range.
         """
-        # TODO remove edge case handling for empty range, now handled separately at init
         byte_range = validate_range(byte_range=byte_range, allow_empty=True)
-        # Do not request an empty range if total length already checked
+        # Do not allow a non-single request async RangeStream to be created
+        if self.client_is_async:
+            raise ValueError("Add a range to an async RangeStream via add_async")
+        # Do not request an empty range if total length already checked (at init)
         if not self._length_checked and byte_range.isempty():
-            self.send_head_request()
+            if self.single_request:
+                self.get_monostream()
+            else:
+                self.send_head_request()
         elif not byte_range.isempty():
-            req = self.send_request(byte_range)
-            if not self._length_checked:
-                self.set_length(req.total_content_length)
-            if byte_range in ranges_in_reg_order(self.ranges):
-                pass  # trivial no-op when adding a range that already exists
-            elif not byte_range.isempty():
-                # bytes are available in the RangeRequest.response stream
-                resp = RangeResponse(stream=self, range_request=req, range_name=name)
-                self.register_range(
-                    rng=byte_range,
-                    value=resp,
-                    activate=activate,
-                )
+            if self.single_request:
+                self.add_window(byte_range=byte_range, activate=activate, name=name)
+            else:
+                req = self.send_request(byte_range=byte_range)
+                if not self._length_checked:
+                    self.set_length(length=req.total_content_length)
+                if byte_range in ranges_in_reg_order(self.ranges):
+                    pass  # trivial no-op when adding a range that already exists
+                elif not byte_range.isempty():
+                    # bytes are available in the RangeRequest.response stream
+                    resp = RangeResponse(
+                        stream=self, range_request=req, range_name=name
+                    )
+                    self.register_range(
+                        rng=byte_range,
+                        value=resp,
+                        activate=activate,
+                        use_windows=False,
+                    )
+
+    @property
+    def is_closed(self):
+        """
+        True if the ``httpx.Response`` object(s) associated with the
+        :class:`~range_streams.response.RangeResponse` values in the internal
+        :attr:`~range_streams.stream.RangeStream._ranges` :class:`~ranges.RangeDict`
+        is/are all closed.
+        """
+        return all(range_response.is_closed for range_response in self._ranges.values())
+
+    def close(self):
+        """
+        Close any ``httpx.Response`` on the stream. In single request mode, there is
+        just the one (shared with all the 'windowed' responses).
+        """
+        for range_response in self._ranges.values():
+            range_response.close()
+
+    @classmethod
+    def make_async_fetcher(
+        cls,
+        urls: list[str],
+        callback: Callable | None = None,
+        verbose: bool = False,
+        show_progress_bar: bool = True,
+        timeout_s: float = 5.0,
+        client=None,
+        **kwargs,
+    ):
+        return AsyncFetcher(
+            stream_cls=cls,
+            urls=urls,
+            callback=callback,
+            verbose=verbose,
+            show_progress_bar=show_progress_bar,
+            timeout_s=timeout_s,
+            client=client,
+            **kwargs,  # Any other kwargs can be passed through to RangeStream subclass
+        )
